// Written on August 29, 2001 by Zachary Deretsky

#include <windows.h> 
#include <stdlib.h> 
#include <stdio.h> 
#include <winbase.h> 
#include <fcntl.h>
#include <io.h>

#define TIME_UNITS_PER_SEC (10000000)

void usage(char* progName)
{
  fprintf (stderr, "usage: %s file_name creation_shift access_shift modification_shift\n", progName);
  fprintf (stderr, "        shifts are in seconds and can be negative\n");
  exit(1);
}

// Time modification as recommended by the MS help 
void timeShift(FILETIME* pInTime, FILETIME* pOutTime, int shift)
{
  LARGE_INTEGER li;

  li.LowPart = pInTime->dwLowDateTime;
  li.HighPart = pInTime->dwHighDateTime;
  li.QuadPart += Int32x32To64(shift, TIME_UNITS_PER_SEC);
  pOutTime->dwLowDateTime = li.LowPart;
  pOutTime->dwHighDateTime =li.HighPart;
}

int main(int argc, char **argv)
{
  if(5 != argc) {
    usage(argv[0]);
  }

  // Open the file
  HANDLE hFile;
  hFile = CreateFile( argv[1], GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
  if(INVALID_HANDLE_VALUE == hFile) {
    fprintf(stderr, "Could not open file: %08x\n", GetLastError());
    exit(1);
  }

  // Obtain file times
  FILETIME cFileTime;
  FILETIME aFileTime;
  FILETIME mFileTime;
  BOOL retVal;

  retVal = GetFileTime(hFile, &cFileTime, &aFileTime, &mFileTime);
  if(!retVal) {
    fprintf(stderr, "Could not get times: %08x\n", GetLastError());
    exit(1);
  }    

  // Read the shift values and shift the times
  int cShift, aShift, mShift;
  cShift = atoi(argv[2]);
  aShift = atoi(argv[3]);
  mShift = atoi(argv[4]);

  FILETIME cNewFileTime;
  FILETIME aNewFileTime;
  FILETIME mNewFileTime;

  timeShift(&cFileTime, &cNewFileTime, cShift);
  timeShift(&aFileTime, &aNewFileTime, aShift);
  timeShift(&mFileTime, &mNewFileTime, mShift);

  // Set the date via the filedate system call and return. Failing
  // this call implies the new file times are not supported by the
  // underlying file system. File times are supported on FAT and NTFS.

  retVal = SetFileTime(hFile, &cNewFileTime, &aNewFileTime, &mNewFileTime);
  if(!retVal) {
    fprintf(stderr, "Could not set times: %08x\n", GetLastError());
    exit(1);
  }    

  CloseHandle( hFile );
  return(0);
}
